import sys
import pathlib
import os
import shutil

root_path = pathlib.Path(__file__).parent.resolve()

def cli():
    if len(sys.argv) > 1:
        print("Initializing django project.... This may take a while")
        try:
            path = sys.argv[1]
            folder_path = os.path.join(root_path, "django-template")

            if path == "." or path == "./":
                path = os.getcwd()

            shutil.copytree(folder_path, path, dirs_exist_ok=True)

        except Exception as e:
            print(e)
    else:
        print("Please provide a path to create the project")



from django.shortcuts import render, redirect
from .forms import CreateUserForm
from django.contrib.auth import authenticate, login, logout
from django.views.generic import TemplateView

class index(TemplateView):
    template_name = 'app/index.html'

class about(TemplateView):
    template_name = 'app/about.html'

class signup(TemplateView):
    template_name = 'app/signup.html'
    def get(self, request):
        if request.user.is_authenticated:
            return render(request, "app/index.html")
        form = CreateUserForm()
        return render(request, self.template_name, {'form': form})

    def post(self, request):
        form = CreateUserForm(request.POST)
        if form.is_valid():
            user = form.save()
            login(request, user)
            return render(request, "app/thanks.html", {'form': form})

        return render(request, self.template_name, {'form': form})

class loginPage(TemplateView):
    template_name = 'app/login.html'

    def get(self, request):
        if request.user.is_authenticated:
            return redirect('index')
        return render(request, self.template_name)

    def post(self, request):
        username = request.POST.get('username')
        password = request.POST.get('password')
        user = authenticate(request, username=username, password=password)
        if user is not None:
            login(request, user)
            return redirect('index')
        else:
            return render(request, self.template_name, {'error_message': 'Username or password is incorrect'})

class logoutPage(TemplateView):
    template_name = 'app/logout.html'

    def get(self, request):
        logout(request)
        return redirect('index')


from django.urls import path, include
from .api_router import router
from .views import index, about, loginPage, signup, logoutPage

urlpatterns = [
    path('', index.as_view(), name="index"),
    path('about/', about.as_view(), name="about"),
    path('login/', loginPage.as_view(), name="loginPage"),
    path('signup/', signup.as_view(), name="signup"),
    path('logout/', logoutPage.as_view(), name="logout"),
    path('api/', include(router.urls)),
]



from rest_framework import viewsets
from .serializers import MyUserSerializer
from rest_framework.permissions import IsAdminUser

from .models import MyUser

class MyUserViewSet(viewsets.ModelViewSet):
    permission_classes = [IsAdminUser]

    queryset = MyUser.objects.all()
    serializer_class = MyUserSerializer


from rest_framework import serializers
from .models import MyUser

class MyUserSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = MyUser
        fields = ['url', 'username', 'email', 'is_staff']


from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _

from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, BaseUserManager

class CustomAccountManager(BaseUserManager):
    # This class let us define our own custom sign up and sign in methods.

    def create_user(self, email, username, password, **extra_fields):
        if not email:
            raise ValueError(_('The Email must be set'))

        email = self.normalize_email(email)
        user = self.model(email=email, username=username, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, email, username, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError(_('Superuser must have is_staff=True.'))
        if extra_fields.get('is_superuser') is not True:
            raise ValueError(_('Superuser must have is_superuser=True.'))

        return self.create_user(email, username, password, **extra_fields)

        pass

class MyUser(AbstractBaseUser, PermissionsMixin):
    # Note: You can'T call the model User as this is a built-in model.
    username = models.CharField(max_length=100, unique=True)
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    email = models.EmailField(unique=True)
    password = models.CharField(max_length=100)
    phone_number = models.CharField(max_length=100)

    is_staff = models.BooleanField(_('staff'), default=False)
    is_active = models.BooleanField(_('active'), default=True)

    # This tells django to use the email as the main sign in field.
    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = [
        'first_name',
        'last_name',
        "username"
    ]

    objects = CustomAccountManager()

    def __str__(self):
        return self.first_name + ' ' + self.last_name

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')


from django.forms import ModelForm
from django.contrib.auth.forms import UserCreationForm
from django import forms
from .models import MyUser

class CreateUserForm(UserCreationForm):
    class Meta:
        model = MyUser
        fields = ['username', 'email', 'password1', 'password2']

    username = forms.CharField(label='Username', max_length=30, required = True, widget=forms.TextInput(attrs={'class': 'form-control'}))
    email = forms.EmailField(label='Email', widget=forms.TextInput(attrs={'class': 'form-control'}))
    password1 = forms.CharField(label='Password', widget=forms.PasswordInput(attrs={'class': 'form-control'}))
    password2 = forms.CharField(label='Password Confirmation', widget=forms.PasswordInput(attrs={'class': 'form-control'}))